/** \file iso_base_notifier.h
 * \class iso::base::Notifier abstract notifier class
 *	\brief abstract notifier class
 *  
 *  complement to the listener class. Notifies any registered listeners that is has changed. 
 *  
 *  Created by Daniel Bisig on 7/22/07.
 */

#ifndef _iso_base_notifier_h_
#define _iso_base_notifier_h_

#include <QVector>

namespace iso
{

namespace base
{

class Listener;

class Notifier
{
public:
	/**
	\brief default constructor
	*/
	Notifier();
	
	/**
	\brief destructor
	*/
	virtual ~Notifier();
	
	/**
	\brief register listener
	\param pListener listener to register
	
	this method doesn't check whether a listener is registered several times
	*/
	void registerListener(Listener* pListener);

	/**
	\brief unregister listener
	\param pListener listener to unregister
	*/
	void unregisterListener(Listener* pListener);
	
	/**
	\brief notify listeners
	*/
	virtual void notifyListeners();
	
	QVector<Listener*>& listeners();
	
protected:
	QVector<Listener*> mListeners;
};

};

};

#endif