/** \file iso_base_types.h
 * \brief all kinds of iso relevant typedefs
 *
 * Defines a variety of variable types for use in all iso classes. In addition, it contains some global defines such as _iso_debug_ etc. Therefore, this header should be included in all iso classes.
 *
 * Created by Daniel Bisig on 05/17/06.
 */

#ifndef _iso_base_types_h_
#define _iso_base_types_h_

#include <QString>
#include <QFile>
#include <sys/time.h>
#include <inttypes.h>
#include <unistd.h>

#define _iso_debug_
//#define _iso_ppc_

namespace iso
{

namespace base
{

/** \brief real variable type
*
* The real variable type. Change this for higher accuracy to double.
*/
typedef float real;

typedef int8_t int8;
typedef uint8_t uint8;
typedef int16_t int16;
typedef uint16_t uint16;
typedef int32_t int32;
typedef uint32_t uint32;	
typedef int64_t int64;
typedef uint64_t uint64;


/** \brief String type
*/
typedef QString String;
	
/**
\brief QFile File
*/
typedef QFile File;
		
/**
\brief timeval struct
*/
typedef struct timeval Time;

/**
\brief timetone struct
*/
typedef struct timezone TimeZone;
};

};

#endif