/**	\file iso_com_message_value_group.h
 *
 *  Created by Daniel Bisig on 14/05/09.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_com_message_value_group_h_
#define _iso_com_message_value_group_h_

#include "iso_com_types.h"
#include "iso_com_exception.h"
#include <iso_base/iso_base_exception.h>
#include <QVector>
#include <QByteArray>
#include <typeinfo>

namespace iso
{

namespace com
{

class MessageValueGroup
{

public:
	template<class Type> MessageValueGroup( unsigned int pValueCount, Type* pValues ) throw (ComException);
	MessageValueGroup( const MessageValueGroup& pValueGroup );
	~MessageValueGroup();
	
	inline const std::type_info* valueType() const;
	inline char valueSize() const;
	inline unsigned int valueCount() const;
	inline unsigned int byteSize() const;
	
	template<class Type> bool checkValues() const;
	template<class Type, int ValueCount> bool checkValues() const;
	template<class Type> const Type* values() const throw (ComException);
	char* buffer() const;

	/**
	\brief print textual value group information 
	*/
	base::String info() const;
	
	/**
	\brief print value group information
	\param pOstream output stream
	\param pMessage message
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const MessageValueGroup& pValueGroup)
	{
		pOstream << pValueGroup.info().toStdString(); 
		
		return pOstream;
	}
	
protected:
	MessageValueGroup();
	
	const std::type_info* mValueType;	
	char mValueSize;
	unsigned int mValueCount;
	unsigned int mByteSize;
	char* mValueBuffer;	
};

template<class Type> 
MessageValueGroup::MessageValueGroup( unsigned int pValueCount, Type* pValues ) throw (ComException)
	: mValueCount(pValueCount)
{
	mValueType = &typeid( Type );
	mValueSize =  sizeof( Type );
	mByteSize = mValueSize * mValueCount;
	mValueBuffer = new char[mByteSize];
	char* pCharValues = (char*)pValues;
	memcpy( mValueBuffer, pCharValues, mByteSize );
}

const std::type_info*  
MessageValueGroup::valueType() const
{
	return mValueType;
}

char 
MessageValueGroup::valueSize() const
{
	return mValueSize;
}

unsigned int 
MessageValueGroup::valueCount() const
{
	return mValueCount;
}

unsigned int 
MessageValueGroup::byteSize() const
{
	return mByteSize;
}

template<class Type> 
bool 
MessageValueGroup::checkValues() const
{
	if( &typeid(Type) == mValueType ) return true;
	return false;
}

template<class Type, int ValueCount> 
bool 
MessageValueGroup::checkValues() const
{
	if( &typeid(Type) == mValueType && mValueCount == ValueCount) return true;
	return false;
}

template<class Type> 
const Type* 
MessageValueGroup::values() const throw (ComException)
{
	if( &typeid(Type) != mValueType ) throw( ComException( "ERR_MESSAGE_VALUE_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(Type).name() ).arg( mValueType->name() ), __FILE__, __FUNCTION__, __LINE__ ));
	return ( (Type*)( mValueBuffer ) );
}

};

};

#endif
