/**	\file iso_com_sender.h
 *  \class iso::com::Sender sends data
 *	\brief send data
 *
 *  Created by Daniel Bisig on 21/5/09.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_com_sender_h_
#define _iso_com_sender_h_

#include "iso_com_types.h"
#include "iso_com_message.h"
#include <QThread>
#include <QMutex>
 
namespace iso
{

namespace com
{

class Receiver;
class MessageParser;
class NetworkSenderProtocol;

class Sender : public QThread
{
public:

	/**
	\brief parameter based constructor
	\param pName sender name
	\param pReceiver receiver for direct communication
	*/
	Sender(const base::String& pString, Receiver* pDirectReceiver);

	/**
	\brief parameter based constructor
	\param pName sender name
	\param pMessageParser message parser
	\param pNetworkProtocol network protocol
	*/
	Sender(const base::String& pName, MessageParser* pMessageParser, NetworkSenderProtocol* pNetworkProtocol );
	
	/**
	\brief desctructor
	*/
	~Sender();
	
	/**
	\brief return sender name
	\return sender name
	*/
	const base::String& name() const;

	/**
	\brief return message parser
	\return message parser
	*/
	const MessageParser* parser() const;
	
	/**
	 \brief return network protocol
	 \return network protocol
	 */
	const NetworkSenderProtocol* protocol() const;	

	/**
	\brief queue message for later sending
	\param pMessage message
	*/
	void queue( const Message& pMessage );

	/**
	\brief send queued messages
	*/
	void send( );
	
	/**
	\brief send message
	\param pMessage message
	\remarks clears existing message queue
	*/
	void send( const Message& pMessage );
	
	/**
	\brief thread run
	*/
	void run();
	
protected:
	/**
	\brief default constructor
	*/
	Sender();

	base::String mName; ///\brief name
	QVector< Message > mMessageQueue; ///\brief message queue
	Receiver* mDirectReceiver; ///\brief direct message receiver
	MessageParser* mMessageParser; ///\brief message parser
	NetworkSenderProtocol* mNetworkProtocol; ///\brief network message protocol
	QMutex mLock; ///\brief sender blocked
};

};

};


#endif