/** \file iso_com_udp_receiver_protocol.h
 *
 *  Created by Daniel Bisig on 5/21/09.
 */

#ifndef _iso_com_udp_receiver_protocol_h_
#define _iso_com_udp_receiver_protocol_h_

#include "iso_com_network_receiver_protocol.h"
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>

namespace iso
{

namespace com
{

class UDPReceiverProtocol : public NetworkReceiverProtocol
{
public:
	/**
	\brief parameter based constructor
	\param pReceiverPort target host port messages are sent to
	\exception ComException failed to create sender
	*/
	UDPReceiverProtocol(unsigned int pReceiverPort) throw (ComException);

	/**
	\brief destructor
	*/
	~UDPReceiverProtocol();
	
	void removeAllBuffers();
	void receive( const MessageParser& pParser ) throw (ComException);
	
protected:
	/**
	\brief default constructor
	*/
	UDPReceiverProtocol();
	
	/**
	\brief remove pending buffers
	*/
	void removePendingBuffers();
	
	int mSocket; ///\brief receiver socket
	struct sockaddr_in mReceiverSocketAddress;
	struct sockaddr_in mSenderSocketAddress;
	char* mReceiveBuffer;
	
	QVector< PendingByteBuffer* > mPendingBuffers; /// \brief pending buffers
	unsigned int mMaxPendingBufferCount; /// \brief maximum number of allowed pending buffers
};

};

};

#endif