/** \file iso_data_wrapper.h
 *  \brief class for wrapping arbitrary data
 *
 *  Class for wrapping arbitrary data. 
 *
 *  Created by Daniel Bisig on 8/2/09.
 *
 */

#ifndef _iso_data_wrapper_h_
#define _iso_data_wrapper_h_

#include "iso_data_types.h"

#include <iso_base/iso_base_exception.h>

#include <iostream>
#include <typeinfo>

namespace iso
{

namespace data
{

class DataWrapper
{
public:
	template <class DataType> DataWrapper( DataType* pDataValue );
	~DataWrapper();

	const std::type_info* type() const;
	template <typename DataType> DataType* data() const throw (base::Exception);
	template <typename DataType> operator DataType*() const throw (base::Exception);

protected:	
	const std::type_info* mDataType; ///\brief data type
	void* mVoidData;
};

template <class DataType>
DataWrapper::DataWrapper( DataType* pDataValue )
{
	mVoidData = static_cast<void*>(pDataValue);
	mDataType = &typeid( *pDataValue );
}

template <typename DataType> 
DataType*
DataWrapper::data() const throw (base::Exception)
{
	if( &typeid(DataType) != mDataType ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(DataType).name() ).arg( mDataType->name() ), __FILE__, __FUNCTION__, __LINE__ ));

	return ( static_cast<DataType*>( mVoidData ) );
}

template <typename DataType> 
DataWrapper::operator DataType*() const throw (base::Exception)
{
	if( &typeid(DataType) != mDataType ) throw( base::Exception( "ERR_TYPE_MISMATCH", base::String("supplied type %1 doesn't match stored type %2").arg( typeid(DataType).name() ).arg( mDataType->name() ), __FILE__, __FUNCTION__, __LINE__ ));

	return ( static_cast<DataType*>( mVoidData ) );
}

};

};

#endif