/** \file iso_geom.h
 *
 *  Created by Daniel Bisig on 4/2/07.
 */

 #ifndef _iso_geom_h_
 #define _iso_geom_h_
 
 #include "iso_geom_types.h"
 #include <iso_math/iso_math_vector2.h>
 #include <iso_math/iso_math_vector3.h>
 #include <QVector>
 
 namespace iso
 {
 
 namespace geom
 {
 
 class Geom
 {
 public:
	static Geom& get();
	static void destroy();
	
	void closestIndex(const math::Vector<real>& pRefPoint, int& pResultIndex, QVector< math::Vector<real> >& pPolyLine, int pStartIndex = -1, int pEndIndex = -1);
	void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResultPoint, QVector< math::Vector<real> >& pPolyLine, int pStartIndex = -1, int pEndIndex = -1);
		
	math::Vector<real> closestPoint(const math::Vector<real>& p_point, QVector< math::Vector<real> >& p_polyLine);
	math::Vector<real> closestPoint(const math::Vector<real>& p_point, QVector< math::Vector<real> >& p_polyLine, unsigned int p_startIndex, unsigned int p_endIndex, unsigned int& p_resultIndex);
 
	/**
	\brief calculate shortest non-self-intersecting path through a group of points
	\param pPoints 3D vectors
	\param pResultIndices result indices into group of 3D vectors
	*/
	void closedPath(const QVector< math::Vector3<real> >& pPoints, QVector< unsigned int>& pResultIndices);
 
 protected:
	Geom();
	~Geom();
	
	static Geom* s_geom;
 
 };
 
 };
 
 };
 
 #endif
