/** \file iso_geom_bezier_spline_alg.h
 *
 *  Created by Daniel Bisig on 4/1/07.
 */

#ifndef _iso_geom_bezier_spline_alg_h_
#define _iso_geom_bezier_spline_alg_h_

#include "iso_geom_spline_alg.h"

namespace iso
{

namespace geom
{

class BezierSplineAlg : public SplineAlg
{
public:
	/**
	\brief create bezier spline
	\param p_dimension dimension of spline 
	\param p_subdivisionCount number of subdivisions
	\param pAutoTangents autotangents mode
	*/
	BezierSplineAlg(unsigned int p_dimension, unsigned int p_subdivisionCount, bool pAutoTangents = true);
	
	/**
	\brief destructor
	*/
	~BezierSplineAlg();
	
	/**
	\brief create a copy of a spline alg
	\return copy of spline alg
	*/
	virtual SplineAlg* copy() const;	

	/**
	\brief set autotangents mode
	\param pAutoTangents autotangents mode
	*/
	void setAutoTangents(bool pAutoTangents);

	/**
	\brief set subdivision count
	\param p_subdivisionCount subdivision count
	*/
	virtual void setSubdivisionCount(unsigned int p_subdivisionCount);

	/**
	\brief create spline
	\param p_controlPoints control points
	\param p_splinePoints resulting spline points
	\param b_closed whether spline should be closed or not
	*/
	virtual void createSpline(QVector< math::Vector<real> >& p_controlPoints, QVector< math::Vector<real> >& p_splinePoints, bool b_closed = false);

protected:
	/**
	\brief default constructor
	*/
	BezierSplineAlg();

	/**
	\brief precalculate polynomial values
	*/
	virtual void preCalc();
	
	/**
	\brief create spline
	\param p_controlPoints control points
	\param p_splinePoints resulting spline points
	\param b_closed whether spline should be closed or not
	*/
	void createAutoTangentSpline(QVector< math::Vector<real> >& p_controlPoints, QVector< math::Vector<real> >& p_splinePoints, bool b_closed = false);	

	/**
	\brief create spline
	\param p_controlPoints control points
	\param p_splinePoints resulting spline points
	\param b_closed whether spline should be closed or not
	*/
	void createExplicitTangentSpline(QVector< math::Vector<real> >& p_controlPoints, QVector< math::Vector<real> >& p_splinePoints, bool b_closed = false);	
	
	/**
	\brief whether tangent are automatically calculated or not
	*/
	bool mAutoTangents;
};

};

};

#endif