/** \file iso_geom_geometry_group.h
 *  \class iso::geom::LineGroup geometry group
 *  \brief line group
 *
 *  Created by Daniel Bisig on 8/2/08.
 */

#ifndef _iso_geom_geometry_group_h_
#define _iso_geom_geometry_group_h_

#include "iso_geom_geometry.h"
#include <QVector>

namespace iso
{

namespace geom
{

class GeometryGroup : public Geometry
{
public:
	/**
	\brief create geometry group
	\param pDim dimension
	*/
	GeometryGroup(unsigned int pDim);

	/**
	\brief destructor
	*/
	~GeometryGroup();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	/**
	\brief return number of geometries
	*/
	unsigned int geometryCount() const;

	/**
	\brief return number of geometrie groups
	*/
	unsigned int geometryGroupCount() const;
	
	/**
	\brief return geometry
	\param pIndex geometry index
	\remarks fails if index is out of bounds
	*/
	Geometry& geometry(unsigned int pIndex);

	/**
	\brief return geometry group
	\param pIndex geometry group index
	\remarks fails if index is out of bounds
	*/
	GeometryGroup& geometryGroup(unsigned int pIndex);

	/**
	\brief return geometries
	\return geometries
	*/
	QVector<Geometry*>& geometries();

	/**
	\brief return geometry groups
	\return geometry groups
	*/
	QVector<GeometryGroup*>& geometryGroups();	
	
	/**
	\brief add geometry to geometry group
	\param pGeometry geometry
	\remarks fails if geometry group and geometry differ with regard to their dimension 
	*/
	void addGeometry(Geometry& pGeometry);

	/**
	\brief add geometry group to geometry group
	\param pGeometryGroup geometry group
	\remarks fails if the two geometry groups differ with regard to their dimension 
	*/
	void addGeometryGroup(GeometryGroup& pGeometryGroup);

	/**
	\brief clear geometry group
	
	remove all geometries and geometry groups
	*/
	void clear();

	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

protected:
	/**
	\brief default constructor
	*/
	GeometryGroup();

	QVector<Geometry*> mGeometries; ///\brief geometries
	QVector<GeometryGroup*> mGeometryGroups; ///\brief geomettry groups
};

};

};

#endif