/** \file iso_geom_line.h
 *
 *  Created by Daniel Bisig on 6/25/08.
 */

#ifndef _iso_geom_line_h_
#define _iso_geom_line_h_

#include "iso_geom_types.h"
#include "iso_geom_geometry.h"
#include <iso_math/iso_math_vector.h>

namespace iso
{

namespace geom
{

class Line : public Geometry
{
public:
	Line();
	Line(unsigned int pDim);
	Line(const math::Vector<real>& pV1, const math::Vector<real>& pV2);
	Line(const Line& pLine);
	~Line();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	inline math::Vector<real>& v0();
	inline math::Vector<real>& v1();

	inline const math::Vector<real>& v0() const;
	inline const math::Vector<real>& v1() const;
	
	void set(unsigned int pVIndex, const math::Vector<real>& pV);
	void set(const math::Vector<real>& pV1, const math::Vector<real>& pV2);
	void set(const Line& pLine);
	const Line& operator=( const Line& pLine );
	
	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

	/**
	\brief return intersection point with line
	\param pLine line 
	\param pResPoint result point
	\return true if there is an intersection, false otherwise
	\remarks fails if dimensions don't match
	*/
	virtual bool intersect( const Line& pLine, math::Vector<real>& pResPoint );

protected:

	math::Vector<real> mV[2];
};

math::Vector<real>& 
Line::v0()
{
	return mV[0];
}

math::Vector<real>&
Line::v1()
{
	return mV[1];
}

const math::Vector<real>& 
Line::v0() const
{
	return mV[0];
}

const math::Vector<real>&
Line::v1() const
{
	return mV[1];
}

};

};

#endif