/** \file iso_geom_plane.h
 *
 *  Created by Daniel Bisig on 12/25/08.
 */

#ifndef _iso_geom_plane_h_
#define _iso_geom_plane_h_

#include "iso_geom_types.h"
#include "iso_geom_geometry.h"
#include <iso_math/iso_math_vector.h>

namespace iso
{

namespace geom
{

class Plane : public Geometry
{
public:
	Plane();
	Plane(unsigned int pDim);
	Plane(const math::Vector<real>& pPoint, const math::Vector<real>& pNormal);
	Plane(const Plane& pPlane);
	~Plane();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	inline math::Vector<real>& point();
	inline math::Vector<real>& normal();

	inline const math::Vector<real>& point() const;
	inline const math::Vector<real>& normal() const;
	
	void setPoint(const math::Vector<real>& pPoint);
	void setNormal(const math::Vector<real>& pNormal);
	void set(const math::Vector<real>& pPoint, const math::Vector<real>& pNormal);
	void set(const Plane& pPlane);
	const Plane& operator=( const Plane& pPlane );

	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

protected:
	math::Vector<real> mPoint;
	math::Vector<real> mNormal;
};

math::Vector<real>& 
Plane::point()
{
	return mPoint;
}

math::Vector<real>&
Plane::normal()
{
	return mNormal;
}

const math::Vector<real>& 
Plane::point() const
{
	return mPoint;
}

const math::Vector<real>&
Plane::normal() const
{
	return mNormal;
}

};

};

#endif