/** \file iso_geom_ray.h
 *
 *  Created by Daniel Bisig on 23/11/08.
 */

#ifndef _iso_geom_ray_h_
#define _iso_geom_ray_h_

#include "iso_geom_types.h"
#include "iso_geom_geometry.h"
#include "iso_geom_line.h"
#include <iso_math/iso_math_vector.h>

namespace iso
{

namespace geom
{

class Ray : public Geometry
{
public:
	Ray();
	Ray(unsigned int pDim);
	Ray(const math::Vector<real>& pO, const math::Vector<real>& pD);
	Ray(const Ray& pRay);
	~Ray();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	inline math::Vector<real>& origin();
	inline math::Vector<real>& direction();

	inline const math::Vector<real>& origin() const;
	inline const math::Vector<real>& direction() const;
	
	void setOrigin(const math::Vector<real>& pOrigin);
	void setDirection(const math::Vector<real>& pDirection);
	void set(const math::Vector<real>& pOrigin, const math::Vector<real>& pDirection);
	void set(const Ray& pRay);
	const Ray& operator=( const Ray& pRay );
	
	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

	/**
	\brief return intersection point with ray
	\param pRay ray 
	\param pResPoint result point
	\return true if there is an intersection, false otherwise
	\remarks fails if dimensions don't match
	*/
	virtual bool intersect( const Ray& pRay, math::Vector<real>& pResPoint );

	/**
	\brief return intersection point with line
	\param pLine line 
	\param pResPoint result point
	\return true if there is an intersection, false otherwise
	\remarks fails if dimensions don't match
	*/
	virtual bool intersect( const Line& pLine, math::Vector<real>& pResPoint );

protected:
	math::Vector<real> mOrigin;
	math::Vector<real> mDirection;
};

math::Vector<real>& 
Ray::origin()
{
	return mOrigin;
}

math::Vector<real>&
Ray::direction()
{
	return mDirection;
}

const math::Vector<real>& 
Ray::origin() const
{
	return mOrigin;
}

const math::Vector<real>&
Ray::direction() const
{
	return mDirection;
}

};

};

#endif