/** \file iso_geom_spline_group.h
 *
 *  Created by Daniel Bisig on 5/11/07.
 */

#ifndef _iso_geom_spline_group_h_
#define _iso_geom_spline_group_h_

#include "iso_geom_geometry.h"
#include "iso_geom_spline.h"
#include <QVector>

namespace iso
{

namespace geom
{

class SplineAlg;

class SplineGroup : public Geometry
{
public:
	/**
	\brief create spline group
	\param pDim dimension
	\param pSplineAlg spline algorithm
	*/
	SplineGroup(unsigned int pDim, SplineAlg* pSplineAlg);

	/**
	\brief copy constructor
	\param pSplineGroup spline group
	*/
	SplineGroup(const SplineGroup& pSplineGroup);

	/**
	\brief destructor
	*/
	~SplineGroup();

	/**
	\brief spline group = spline group
	\param pSplineGroup spline group to copy values from
	\return spline group
	\remark fails if dimensions don't match
	*/
    const SplineGroup& operator=( const SplineGroup& pSplineGroup ); 

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	/**
	\brief return number of splines
	*/
	unsigned int splineCount() const;
	
	/**
	\brief return splines
	\returns splines
	*/
	QVector<Spline*>& splines();
	
	/**
	\brief return spline
	\param pIndex spline index
	\remarks fails if index is out of bounds
	*/
	Spline& spline(unsigned int pIndex);

	/**
	\brief return spline algorithm
	*/
	SplineAlg* splineAlg();	
	
	/**
	\brief add spline to spline group
	\param pSpline spline
	\remarks fails if spline group and spline differ with regard to their dimension 
	*/
	void addSpline(Spline& pSpline);

	/**
	\brief update splines
	*/
	void update();

	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

protected:
	/**
	\brief default constructor
	*/
	SplineGroup();

	/**
	\brief splines
	*/
	QVector<Spline*> mSplines;

	/**
	\brief spline algorithm
	*/
	SplineAlg* mSplineAlg;
};

};

};

#endif