/** \file iso_geom_tesselator.h
 *  \class iso::geom::Tesselator triangulation helper class
 *  \brief triangulation helper class
 *
 *  Created by Daniel Bisig on 08/07/08.
 *
 */

#ifndef _iso_geom_tesselator_h_
#define _iso_geom_tesselator_h_

#include "iso_geom_types.h"
#include "iso_geom_poly_line.h"
#include "iso_geom_mesh.h"
#include <iso_math/iso_math_vector3.h>
#include <QMutex>
#include <QHash>

#ifdef _linux_
#include <GL/glu.h>
#else
#include <OpenGL/glu.h>
#endif

namespace iso 
{

namespace geom 
{

enum TesselationRule 
{
	Odd,
	Nonzero,
	Positive,
	Negative,
	AbsGeqTwo
};

enum TesselationMode
{
	TriangleFan,
	TriangleStrip,
	Triangles,
	LineLoop
};

class Tesselator
{
public:
	void setTesselationRule( TesselationRule pTesselationRule );
	void setTesselationRule( GLenum pTesselationRule );
	void tesselate(  const QVector< PolyLine >& pContours, QVector< math::Vector3<double> >& pTessVertices, QVector< unsigned int*>& pFaceVertexIndices );

	static Tesselator& get();
	static void destroy();

protected:
	static Tesselator* sTesselator;
	//static QVector< GLdouble* > sIntersectPoints; ///\brief points created during tesselation from self intersecting edges	
	
	QVector< math::Vector3<double> >* mTessVertices;
	QVector< int > mTessVertexSequence;
	QVector< unsigned int* > mTessVertexIndices; 
	QVector< unsigned int* >* mTessFaceVertexIndices;
	TesselationMode mTesselationMode;
	
 	Mesh* mMesh;
	
	GLUtesselator *mTessObj; ///\brief the tessellation object
	GLdouble mTesselationRule;
	QHash< TesselationRule, GLdouble > mTesselationRules;
	QMutex mLock; ///\brief tesselator blocks till it finished tesselating
	
	static void beginCallback(GLenum which);
	static void endCallback();
	static void combineCallback(GLdouble coords[3], void *vertex_data[4], GLfloat weight[4], unsigned int **dataOut);
	static void vertexCallback(GLvoid *vertex);

	Tesselator();
	~Tesselator();

	int init();
	int processContour( unsigned int pVertexIndex, unsigned int pVertexCount );
	int beginPolygon();
	int endPolygon();
	int beginContour();
	int endContour();
	int end();
};
	
};

};

#endif