/** \file iso_gl_program.h
 *
 *  Created by Daniel Bisig on 8/12/07.
 */

#ifndef _iso_gl_program_h_
#define _iso_gl_program_h_

#include "iso_gl.h"
#include "iso_gl_types.h"
#include "iso_gl_exception.h"
#include "iso_gl_texture.h"
#include "iso_gl_shader.h"

namespace iso
{

namespace gl
{

class BufferObject;

class Program
{
public:
	/**
	\brief default constructor
	*/
	Program() throw (GLException);
	
	/**
	\brief destructor
	*/
	~Program();

	inline bool linked() const;

	/**
	\brief link program
	*/
	void link() throw (GLException);
	
	/**
	\brief return program id
	\return program id
	*/
	GLuint id() const;
	
	/**
	\brief return variable location
	\param pVariableName name of variable
	\return variable location
	*/
	GLuint variableLocation( const base::String& pVariableName ) throw (GLException);

	GLuint attributeLocation( const base::String& pAttributeName ) throw (GLException);
	
//	template< typename VariableType > void setVariable( const base::String& pVariableName, VariableType pValue ) throw (GLException);
//	template< typename VariableType > void setVariable( const base::String& pVariableName, VariableType pValue1, VariableType pValue2 ) throw (GLException);
//	template< typename VariableType > void setVariable( const base::String& pVariableName, VariableType pValue1, VariableType pValue2, VariableType pValue3 ) throw (GLException);
//	template< typename VariableType > void setVariable( const base::String& pVariableName, VariableType pValue1, VariableType pValue2, VariableType pValue3, VariableType pValue4 ) throw (GLException);

	void setVariable( const base::String& pVariableName, int pValue ) throw (GLException);
	void setVariable( const base::String& pVariableName, int pValue1, int pValue2 ) throw (GLException);
	void setVariable( const base::String& pVariableName, int pValue1, int pValue2, int pValue3 ) throw (GLException);
	void setVariable( const base::String& pVariableName, int pValue1, int pValue2, int pValue3, int pValue4 ) throw (GLException);
	void setVariable( const base::String& pVariableName, int pSize, int* pVectorValues ) throw (GLException);
	
	void setVariable( const base::String& pVariableName, float pValue ) throw (GLException);
	void setVariable( const base::String& pVariableName, float pValue1, float pValue2 ) throw (GLException);
	void setVariable( const base::String& pVariableName, float pValue1, float pValue2, float pValue3 ) throw (GLException);
	void setVariable( const base::String& pVariableName, float pValue1, float pValue2, float pValue3, float pValue4 ) throw (GLException);
	void setVariable( const base::String& pVariableName, int pSize, float* pVectorValues ) throw (GLException);
	
	void setMatrix( const base::String& pMatrixName, int pSize, float* pMatrixValues ) throw (GLException);
	void setMatrix( const base::String& pMatrixname, int pSize1, int pSize2, float* pMatrixValues ) throw (GLException);
	
	void setTexture(  const base::String& pTextureName, Texture& pTexture, GLuint pTextureUnit ) throw (GLException);
	
	void setAttribute( const base::String& pBufferObjectName, const BufferObject* pBufferObject ) throw (GLException);
	void setAttribute( const base::String& pAttributeName, const BufferObject* pBufferObject, int pBufferOffset ) throw (GLException);
	/**
	\brief check if shader is attached to this program
	\return true if shader is already attached to this program, false otherwise
	*/
	bool checkShader(Shader& pShader);
	
	/**
	\brief attach shader to program
	\param pShader shader
	*/
	void attachShader(Shader& pShader) throw (GLException);
	
	/**
	\brief detach shader from program
	\param pShader shader
	*/
	void detachShader(Shader& pShader) throw (GLException);
	
	/**
	\brief enable program
	*/
	void enable();
	
	/**
	\brief disable program
	*/
	void disable();	
	
	/**
	\brief delete program
	*/
	void destroy();
	
	/**
	\brief print shader info log
	*/
	void printInfoLog() const;

protected:
	bool mLinked;
	GLuint mId; ///\brief program id
};

bool 
Program::linked() const
{
	return mLinked;
}
	
//template< typename VariableType > 
//void 
//Program::setVariable( const base::String& pVariableName, VariableType pVariableValue ) throw (GLException)
//{
//	try 
//	{
//		GL::get().clearError();
//		
//		glUniform1i( variableLocation( pVariableName ), pVariableValue );
//		
//		GL::get().error();
//	}
//	catch (GLException& e) 
//	{
//		std::cout << e << "\n";
//		
//		throw( GLException( "ERR_GLSL_PROGRAM", base::String("Failed to Set Variable %1").arg(pVariableName), __FILE__, __FUNCTION__, __LINE__ ) );
//	}
//}
//	
//template< typename VariableType > 
//void 
//Program::setVariable( const base::String& pVariableName, VariableType pVariableValue1, VariableType pVariableValue2 ) throw (GLException)
//{
//	try 
//	{
//		GL::get().clearError();
//			
//		glUniform2i( variableLocation( pVariableName ), pVariableValue1, pVariableValue2 );
//		
//		GL::get().error();
//	}
//	catch (GLException& e) 
//	{
//		std::cout << e << "\n";
//			
//		throw( GLException( "ERR_GLSL_PROGRAM", base::String("Failed to Set Variable %1").arg(pVariableName), __FILE__, __FUNCTION__, __LINE__ ) );
//	}
//}
//
//template< typename VariableType > 
//void 
//Program::setVariable( const base::String& pVariableName, VariableType pVariableValue1, VariableType pVariableValue2, VariableType pVariableValue3 ) throw (GLException)
//{
//	try 
//	{
//		GL::get().clearError();
//		
//		glUniform3i( variableLocation( pVariableName ), pVariableValue1, pVariableValue2, pVariableValue3 );
//		
//		GL::get().error();
//	}
//	catch (GLException& e) 
//	{
//		std::cout << e << "\n";
//		
//		throw( GLException( "ERR_GLSL_PROGRAM", base::String("Failed to Set Variable %1").arg(pVariableName), __FILE__, __FUNCTION__, __LINE__ ) );
//	}
//}
//
//template< typename VariableType > 
//void 
//Program::setVariable( const base::String& pVariableName, VariableType pVariableValue1, VariableType pVariableValue2, VariableType pVariableValue3, VariableType pVariableValue4 ) throw (GLException)
//{
//	try 
//	{
//		GL::get().clearError();
//		
//		glUniform4i( variableLocation( pVariableName ), pVariableValue1, pVariableValue2, pVariableValue3, pVariableValue4 );
//		
//		GL::get().error();
//	}
//	catch (GLException& e) 
//	{
//		std::cout << e << "\n";
//		
//		throw( GLException( "ERR_GLSL_PROGRAM", base::String("Failed to Set Variable %1").arg(pVariableName), __FILE__, __FUNCTION__, __LINE__ ) );
//	}
//}	

};

};

#endif