/** \file iso_gl_types.h
 *
 *  Created by Daniel Bisig on 6/3/07.
 */

 #ifndef _iso_gl_types_h_
 #define _iso_gl_types_h_
 
 #include <iso_base/iso_base_types.h>
 #include <iso_img/iso_img_buffer.h>
 #ifdef _linux_
 #define GL_GLEXT_PROTOTYPES
 #include <GL/glu.h>
 #include <GL/glext.h>
 #else
 #include <OpenGL/gl.h>
 #include <OpenGL/glu.h>
 #include <OpenGL/glext.h>
 #endif

 namespace iso
 {
 
 namespace gl
 {
 
 typedef img::Buffer<unsigned char> ColorBuffer;
 typedef base::real real;

 enum DataType
 {
	Byte = GL_BYTE,
	UByte = GL_UNSIGNED_BYTE,
	Int = GL_INT, 
	UInt = GL_UNSIGNED_INT,
	Float = GL_FLOAT
 };
 
 enum DrawPrimitiveType
 {
	PointPrimitives = GL_POINTS,
	LinePrimitives = GL_LINES,
	LineStripPrimitives = GL_LINE_STRIP,
	LineLoopPrimitives = GL_LINE_LOOP,
	TrianglePrimitives = GL_TRIANGLES,
	TriangleStripPrimitives = GL_TRIANGLE_STRIP,
	TriangleFanPrimitives = GL_TRIANGLE_FAN
 };
 
 enum ColorType
 {
	Alpha = GL_ALPHA,
	Depth = GL_DEPTH_COMPONENT,
	Stencil = GL_STENCIL_INDEX,
	Luminance = GL_LUMINANCE,
	Intensity = GL_INTENSITY,
	RGB = GL_RGB,
	RGBA = GL_RGBA,
	BGRA= GL_BGRA
 };
	  
 enum BufferObjectType
 {
	ArrayBuffer = GL_ARRAY_BUFFER,
	IndexBuffer = GL_ELEMENT_ARRAY_BUFFER
 };

 enum BufferUsageType
 {
	StaticDraw = GL_STATIC_DRAW,
	DynamicDraw = GL_DYNAMIC_DRAW
 };
 
 inline GLenum glType( char pType ) { return GL_BYTE; }
 inline GLenum glType( unsigned char pType ) { return GL_UNSIGNED_BYTE; }
 inline GLenum glType( int pType ) { return GL_INT; }
 inline GLenum glType( unsigned int pType ) { return GL_UNSIGNED_INT; }
 inline GLenum glType( float pType ) { return GL_FLOAT; }
 template< typename Type > GLenum glType( ) 
 { 
	Type dt;
	return glType( dt ); 
 }
 
 inline DataType dataType( char pType ) { return Byte; }
 inline DataType dataType( unsigned char pType ) { return UByte; }
 inline DataType dataType( int pType ) { return Int; }
 inline DataType dataType( unsigned int pType ) { return UInt; }
 inline DataType dataType( float pType ) { return Float; }
 template< typename Type > DataType dataType( ) 
 { 
	Type dt;
	return dataType( dt ); 
 }
 
 inline ColorType colorType( GLenum pColorType  )
 {
	if( pColorType == GL_ALPHA ) return Alpha;
	if( pColorType == GL_DEPTH_COMPONENT ) return Depth;
	if( pColorType == GL_LUMINANCE ) return Luminance;
	if( pColorType == GL_INTENSITY ) return Intensity;
	if( pColorType == GL_RGB ) return RGB;
	if( pColorType == GL_RGBA ) return RGBA;
 }

  
 };
 
 };
 
 #endif