/** \file iso_math_field_algorithm.h
 * \class iso::math::FieldAlgorithm abstract field algorithm base class
 * \brief abstract field algorithm base class
 *
 *  Created by Daniel Bisig on 8/31/07.
 */

#ifndef _iso_math_field_algorithm_h_
#define _iso_math_field_algorithm_h_

#include "iso_math_vector.h"
#include "iso_math_vector_field.h"

namespace iso
{

namespace math
{

class FieldAlgorithm
{
friend class VectorField<real>;

public:
	FieldAlgorithm();
	FieldAlgorithm(unsigned int pFieldDim, unsigned int pVectorDim);
	FieldAlgorithm(const Vector<real>& pMinParValue, const Vector<real>& pMaxParValue, unsigned int pVectorDim) throw (MathException);
	virtual ~FieldAlgorithm();
	
	void setVectorField(VectorField<real>* pVectorField) throw (MathException);
	
	virtual void update() = 0;
	
	void resetTime(real pTime);
	void setTimeStep(real pTimeStep);
	void setTimeLimit(real pTimeLimit);
	
protected:
	static unsigned int sFieldDim; /// \brief default field dimension
	static unsigned int sVectorDim; /// \brief default vector dimension
	static real sTime;
	static real sTimeStep;
	static real sTimeLimit;

	unsigned int mFieldDim; /// \brief field dimension
	unsigned int mVectorDim; /// \brief vector dimension
	Vector<real> mMinParValue; /// \brief minimum function parameter value
	Vector<real> mMaxParValue; /// \brief maximum function parameter value
	Vector<real> mParValueStep; /// \brief function parameter step size
	
	real mTime;
	real mTimeStep;
	real mTimeLimit;
	
	VectorField<real>* mVectorField;
};

};

};

#endif

