/**	\file iso_serialize_parser.h
 *  \class iso::serialize::SerializeParser singleton class for parsing serialization files
 *	\brief singleton class for parsing serialization files
 * 
 *  Created by Daniel Bisig on 8/19/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef iso_config_parser_h_ 
 #define iso_config_parser_h_
 
 #include "iso_serialize_types.h"
 #include "iso_serialize_data.h"
 #include <iso_xml/iso_xml_tools.h>
 
 namespace iso
 {
 
 namespace serialize
 {
 
 class SerializeParser
 {
 public:
	/**
	\brief return singleton instance
	\return singleton instance
	*/
	static SerializeParser& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();


	/**
	\brief read serialization file
	\param pFileName serialization file to read from
	\Exception SerializeException failed to parse serialization file
	*/
	SerializeData read(const base::String& pFileName) throw (SerializeException);
	
	/**
	\brief write serialization file
	\param pSerializeData SerializaData
	\param pFileName serialization file to write to
	\param pFileType type of serialization file
	\param impl DOM implementation
	\return DOM document
	*/
	xercesc::DOMDocument* write(const SerializeData& pSerializeData, const base::String& pFileName, const base::String& pFileType, xercesc::DOMImplementation *impl);
 
 protected:
	/**
	\brief default constructor
	*/
	SerializeParser();
	
	/**
	\brief destructor
	*/
	virtual ~SerializeParser(); 
 
 	/**
	\brief singleton class instance
	*/
	static SerializeParser* sSerializeParser;

	/**
	\brief recursively convert SerializeData object into DOM element
	\param pSerial SerializeData
	\param pElement DOM Element
	\param pDoc DOM Document
	*/
	void parseSerial(const SerializeData& pSerial, xercesc::DOMElement* pElement, xercesc::DOMDocument* pDoc);
	
	/**
	\brief recursively convert DOM element into SerializeData object
	\param pElement DOM element
	\return SerializeData
	*/
	SerializeData parseNode(xercesc::DOMElement* pElement);
 };
 
 };
 
 };
 
 #endif