/** \file iso_space_alg_kdtree.h
 *
 *  Created by Daniel Bisig on 6/8/09.
 */
 
#ifndef _iso_space_alg_kdtree_h_
#define _iso_space_alg_kdtree_h_

#include "iso_space_alg.h"
#include <kdtree.h>

namespace iso
{

namespace space
{

class KDTreeAlg : public SpaceAlg
{
public:
	KDTreeAlg(unsigned int pDim);
	KDTreeAlg(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);
	~KDTreeAlg();

	void updateStructure( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	void updateNeighbors( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	
	/**
	\brief obtain textual ntree information
	\return String containing textual ntree information
	*/
	base::String info() const;

	/**
	\brief retrieve textual ntree information
	\param pOstream output stream
	\param pTree ntree
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const KDTreeAlg& pAlg)
	{
		pOstream << pAlg.info().toStdString(); 
		
		return pOstream;
	}

protected:
	KDTreeAlg();
	
	kdtree* mTree; ///\brief KD Tree
	kdres* mSearchResult; ///\brief Search Result
};

};

};

#endif