/** \file iso_space_alg_rtree.h
 *
 *  Created by Daniel Bisig on 6/9/09.
 */
 
#ifndef _iso_space_alg_rtree_h_
#define _iso_space_alg_rtree_h_

#include "iso_space_types.h"
#include "iso_space_alg.h"
#include "iso_space_rtree.h"

namespace iso
{

namespace space
{

class RTreeAlg : public SpaceAlg
{
public:
	RTreeAlg(unsigned int pDim);
	RTreeAlg(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);
	~RTreeAlg();
	
	ClosestShapePointType closestShapePointType() const;
	

	/**
	\brief set closest point type
	\param pClosestPointType closest point type
	*/
	void setClosestPointType(ClosestShapePointType pClosestPointType);

	void updateStructure( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	void updateNeighbors( QVector< SpaceProxyObject* >& pObjects ) throw (SpaceException);
	
	/**
	\brief obtain textual ntree information
	\return String containing textual ntree information
	*/
	base::String info() const;

	/**
	\brief retrieve textual ntree information
	\param pOstream output stream
	\param pTree ntree
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const RTreeAlg& pAlg)
	{
		pOstream << pAlg.info().toStdString(); 
		
		return pOstream;
	}

protected:
	RTreeAlg();

	/**
	\brief RTree space partitioning instance
	*/
	RTree<SpaceProxyObject*, real, real> mTree;

	ClosestShapePointType mClosestPointType;	
};

};

};

#endif