/** \file iso_space_grid_tools.h
 *
 *  Created by Daniel Bisig on 9/23/07.
 */

#ifndef _iso_space_grid_tools_h_
#define _iso_space_grid_tools_h_

#include "iso_space_grid.h"
#include "iso_space_mesh_shape.h"
#include <iso_math/iso_math_vector3.h>
#include <iso_geom/iso_geom_geometry_group.h>
#include <iso_data/iso_data_buffer.h>
#include <opencv/cv.h>
#include <opencv/cxcore.h>

namespace iso
{

namespace space
{

enum GridDistanceMode
{
	ShapeDistance,
	ContourDistance,
	ShapeTangent,
	ContourTangent
};

class SpaceGridTools
{
public:
	/**
	\brief obtain singleton instance
	\return singleton instance
	*/
	static SpaceGridTools& get();

	/**
	\brief destroy singleton instance
	*/
	static void destroy();
	
	/**
	\brief create space grid from file
	\return space grid
	
	not yet implemented!!
	*/
	SpaceGrid createGrid(const base::String& pFileName);
	
	/**
	\brief create distance space grid from mesh shape
	\param pGridSubdivisionCount number of space grid subdivisions
	\param pGridMinPos minimum space grid position
	\param pGridMaxPos maximum space grid position
	\param pMeshShape mesh shape
	\return space grid
	
	calculates distance field in world coordinates.\n
	the distance vectors point from individual grid positions towards to closest position on the mesh shape's triangulated surface
	*/
	SpaceGrid createDistanceGrid( const Vector3<unsigned int>& pGridSubdivisionCount, const math::Vector3<real>& pGridMinPos, const math::Vector3<real>& pGridMaxPos, MeshShape& pMeshShape );

	/**
	\brief create distance space grid for space shape
	\param pSpaceGrid space grid
	\param pShape shape
	\remark fails if dimensions don't match
	
	calculates distance field in world coordinates.\n
	the distance vectors point from individual grid positions towards to closest position to the space shape object
	*/
	void createDistanceGrid( SpaceGrid& pSpaceGrid, SpaceShape& pShape );

	/**
	\brief create distance space grid for space shape
	\param pSpaceGrid space grid
	\param pShape shape
	\param pScale distance scale
	\remark fails if dimensions don't match
	
	calculates distance field in world coordinates.\n
	the distance vectors point from individual grid positions towards to closest position to the space shape object
	*/
	void createDistanceGrid( SpaceGrid& pSpaceGrid, SpaceShape& pShape, math::VectorField<real>& pScale );

protected:
	/**
	\brief default constructor
	*/
	SpaceGridTools();
	
	/**
	\brief destructor
	*/
	~SpaceGridTools();

	static SpaceGridTools* sSpaceGridTools; /// \brief singleton instance
};

};

};

#endif